


#ifndef FILE_UTIL_H
#define FILE_UTIL_H

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>
#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <exception>

using boost::filesystem::path;
namespace fs = boost::filesystem;

typedef std::string Extent;
typedef std::vector<Extent> Extents;


inline Extents buildExtents(char const * e1 = "",char const * e2 = "",char const * e3 ="",
					 char const * e4 = "" ,char const * e5 = "") {

	Extents exts;

	if(strcmp(e1 , "")) {

		exts.push_back(std::string(e1));
	}

	if(strcmp(e2 , "")) {

		exts.push_back(std::string(e2));
	}
	
	if(strcmp(e3 , "")) {

		exts.push_back(std::string(e3));
	}
	
	if(strcmp(e4 , "")) {

		exts.push_back(std::string(e4));
	}
	
	if(strcmp(e5 , "")) {

		exts.push_back(std::string(e5));
	}



	return exts;
}

class File {
	
	boost::filesystem::path _filePath;
public:
	
	explicit File(boost::filesystem::path filePath)
		:_filePath( boost::filesystem::complete (filePath) )   {

		if(!isFile(filePath)) {
			std::string temp = filePath.string() + std::string("is Not File");
		//	throw std::exception(temp.c_str());
		}

	}

	bool const operator< (File const &fil) const {
		return _filePath.string() < fil._filePath.string ();
	}

	bool const operator== (File const &fil) const {
		return _filePath.string()== fil._filePath.string ();
	}

	std::string const string() const  {
		return _filePath.string ();
	}


	std::string const leaf() const  {
		return _filePath.leaf ();
	}

	boost::filesystem::path  const branch_path() const {
		return _filePath.branch_path();
	}




	static bool const isFile(boost::filesystem::path filePath) {

	
		if(!boost::filesystem::exists(filePath)) {			
			return false;
		}

		if(boost::filesystem::is_directory (filePath)) {
			return false;
		}

				

		return true;



	}


	
	 bool is_ext( Extent const  & ext) const  {

		if(ext.size() == 0) {
			return false;
		}

		std::string filename = _filePath.leaf();
		if(filename.size() < ext.size() + 2 ) {
			return false;
		}

		size_t start = filename.size() - ext.size();

		if(filename[ start - 1] != '.'){
			return false;
		}


		for(size_t index = 0;index < ext.size(); ++index){
			if(filename[start + index ] != ext[ index ]) {
				return false;
			}
		}

		return true;

	}


	bool is_ext( Extents  const& ext) const  {

		for(size_t i = 0; i < ext.size(); ++i ) {
			if(is_ext(ext[i])){
				return true;
			}
		}

		return false;

	}


	boost::filesystem::path getPath() const {

		return _filePath;

	}
	


};


class LegalFile :public File {

};


class LegalDirectory {
	boost::filesystem::path _dir;

	public:

	struct NotLegalDirectory : public std::exception {


		boost::filesystem::path _errPath;
		

	
		NotLegalDirectory(boost::filesystem::path const & errPath)
			:_errPath(errPath) {
		}

		virtual const char * what() const throw(){

			using namespace std;

			static std::string errString 
				= //string(exception::what()) +
				string("Is Not LegalDirectoryPath  path : ") +  _errPath.string ();

			return errString.c_str();

		}

		~NotLegalDirectory() throw(){
		}




	};


	LegalDirectory(boost::filesystem::path const & dir)
		:_dir(dir) {

		if(boost::filesystem::exists(dir) 
			&& boost::filesystem::is_directory(dir) 
			&&(dir.leaf() != std::string("cvs")) 
			&&(dir.leaf() != std::string("cvS"))
			&&(dir.leaf() != std::string("cVs"))
			&&(dir.leaf() != std::string("cVS"))
			&&(dir.leaf() != std::string("Cvs"))
			&&(dir.leaf() != std::string("CvS"))
			&&(dir.leaf() != std::string("CVs"))
			&&(dir.leaf() != std::string("CVS"))
			) {
			//ok nothrow;
		}else {
			throw NotLegalDirectory(dir);
		}

	}

	boost::filesystem::path const getPath() const{
		return _dir;
	}

};


typedef std::set<File> FileList;

struct FileUtil {

		
	static void gather_recusive(LegalDirectory pa, Extents const &ext,FileList &result_files ) {
		for(size_t i = 0; i < ext.size(); ++i ) {
			gather_recusive(pa,ext[i], result_files);	
		}
	}

	static void gather_recusive(LegalDirectory pa, Extent const &ext,FileList &result_files ) {

		

		fs::directory_iterator end_itr;

		for(fs::directory_iterator dir_itr (pa.getPath()) ; dir_itr != end_itr; ++dir_itr) {

			fs::path subPa = *dir_itr;
			if(fs::is_directory(subPa) ) {
					if(dir_itr->leaf() != std::string("cvs")
					&& dir_itr->leaf() != std::string("cvS")
					&& dir_itr->leaf() != std::string("cVs")
					&& dir_itr->leaf() != std::string("cVS")
					&& dir_itr->leaf() != std::string("Cvs")
					&& dir_itr->leaf() != std::string("CvS")
					&& dir_itr->leaf() != std::string("CVs")
					&& dir_itr->leaf() != std::string("CVS")


				) {
					gather_recusive(subPa,ext,result_files);

				}

			}else {
				File file(subPa);
				if(file.is_ext (ext) ){
					result_files.insert(file);
				}
				else {
					//none;
				}
			}
		}    


		

	}



	static FileList filtFile(Extents const & ext,FileList const & fileList ) {

		FileList result;

		for(FileList::const_iterator iter = fileList.begin(); iter != fileList.end(); ++iter) {

			if( iter->is_ext(ext)){
                result.insert(*iter);
			}
		}

		return result;


	}



};

#endif //FILE_UTIL_H



