
#ifndef PROJECT_H_
#define PROJECT_H_

#include "PathUtil.h"
#include "FileUtil.h"


class ProjectFactory;
class ProjectDebugFactory;
class ProjectReleaseFactory;


class Project{

	LegalDirectory _prjPath;
	std::string  _leafString ;
	std::string  _prjName;
	
	Project(LegalDirectory const &prjPath, std::string  const& leafString )
		:_prjPath(boost::filesystem::complete(prjPath.getPath()) )
		,_leafString(leafString)
		,_prjName(prjPath.getPath().leaf() ) {		

	}

public:

	

	LegalDirectory getMakefileDir() {
		boost::filesystem::path  temp = _prjPath.getPath() / _leafString;

		boost::filesystem::path result = boost::filesystem::complete(temp);

		if(!boost::filesystem::exists(result)) {
			boost::filesystem::create_directory(result);

		}
		return result;

	}

	boost::filesystem::path getMakefileFile() {

		return getMakefileDir().getPath() / ("makefile");

	
	}
	
	
	LegalDirectory getLibraryDir() {
		return getMakefileDir() ;
	}
    
	boost::filesystem::path getLibraryFile() {
		return getLibraryDir().getPath()  / (std::string("lib") + _prjName + ".a");
		
	}



	LegalDirectory getVcprjDir() {
		return getDirectory() ;
	}
    
	boost::filesystem::path getVcprjFile() {
		return getVcprjDir().getPath()  / ( _prjName + ".vcproj");
	}


	std::string getProjectName() {
		return _prjName;

	}


	LegalDirectory getDirectory() {
		return _prjPath;
	}




	
	
	

	friend class ProjectFactory;
	friend class ProjectDebugFactory;
	friend class ProjectReleaseFactory;

};


typedef std::vector<Project> ProjectList;



class ProjectFactory {
public:
	virtual Project getProject(LegalDirectory const &prjPath) = 0;
};

class ProjectDebugFactory{

public:
	virtual Project getProject(LegalDirectory const &prjPath) {
		return Project(prjPath,std::string("debug"));

	}

};

class ProjectReleaseFactory {

public:
	virtual Project getProject(LegalDirectory const &prjPath ) {
		return Project(prjPath,std::string("release"));
	}

};



#endif
//PROJECT_H_

