

#include <stdio.h>
#include <string>
#include <vector>

#include <iostream>
#include <fstream>
#include <sys/stat.h>
#include <algorithm>
#include <exception>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>


#include "../Module.h"
#include "../FileUtil.h"
#include "../PathUtil.h"

#include "../DependInfoComputer.h"
#include "../Project.h"



using namespace std;

struct CPPFile {


	static std::string objName(File const & cppFile) {		
		std::string result = cppFile.leaf();
		result.resize(result.size() - 3);
		result += "o";

		return result;

	}	


	static std::string objFullName(File const & cppFile) {
		std::string result = cppFile.string();
		result.resize(result.size() - 3);
		result += "o";

		return result;

	}



};


class  PathMoum {


public:

	static boost::filesystem::path podolibPath() {

		return ("../../podo");
	}
	static boost::filesystem::path podoCustomPath() {

		return ("../../custom");
	}


	static boost::filesystem::path boostPath() {

		return ("../../../boost");
	}

	static boost::filesystem::path examplePath() {
		return ("../../example");
	}


	static std::string xlibPath() {
		return std::string ("/usr/X11R6/lib");
	}

	

};







class CommonWriter {
public:




	static void writeHead (Project & project, std::ofstream &fout_dst) {

		fout_dst<< endl;
		fout_dst<< endl;
		fout_dst<<"INCLUDE = "//<<PathMoum::xlibPath () <<" "
			<<PathUtil::relatePath(PathMoum::boostPath (), project.getMakefileDir().getPath())<<endl;
		//fout_dst<<"LIBS = "<<endl;
		fout_dst<< "CPP = g++ -c"<<endl;
		fout_dst<< "CPPFLAGS = -g -D_DEBUG -D_GCC -D_UNICODE -D_PODO_DEBUG -D_PODO_POSIX"<< endl;
		fout_dst<<endl;
		fout_dst<< "LD = g++"<<endl;
		fout_dst<< "LDFLAGS = "<<endl;


	}


	static void writeOBJS ( FileList &cppFiles,	std::ofstream &fout_dst) {

		
		fout_dst << endl;

		fout_dst << "OBJS = ";
		
		for(FileList::const_iterator iter =  cppFiles.begin(); iter !=  cppFiles.end(); ++iter) {
			fout_dst<<  CPPFile::objName(*iter);
				fout_dst<<"  " ;	
		}
		
		fout_dst << endl;

	}


	
	static void writeClean( std::ofstream &fout_dst) {

		fout_dst <<endl;
		fout_dst <<"clean : "<<endl;
		fout_dst <<'\t'<< "rm ${OBJS} ${TARGET}";

	}
	
	
	static void writeOBJRull ( DependInfo const & dependInfo, 
							FileList &cppFiles,
							Project project , std::ofstream &fout_dst) {


		fout_dst << endl;
		
		for(FileList::const_iterator iter =  cppFiles.begin(); iter !=  cppFiles.end(); ++iter) {

			
				fout_dst<<endl;
				


				fout_dst<<  CPPFile::objName(*iter);
				fout_dst<<" : " ;


				if(dependInfo.end() != dependInfo.find(*iter)) {

					DependInfo::const_iterator  iterTemp = dependInfo.find(*iter) ;
					FileList const &dependList =  iterTemp->second;
		            			
					for(FileList::const_iterator iterDependList = dependList.begin() ;
						iterDependList != dependList.end(); ++iterDependList) {

						path targetPath = iterDependList->branch_path ();

						fout_dst << PathUtil::relatePath  (targetPath, project.getMakefileDir ().getPath() ) << iterDependList->leaf ();
						fout_dst << "    ";

					}
				}

				fout_dst<<std::endl<<"\t"<< "${CPP} ${CPPFLAGS} -I${INCLUDE} " << PathUtil::relatePath  (iter->branch_path(), project.getMakefileDir ().getPath() ) << iter->leaf () ;
				//fout_dst<<std::endl<<"\t"<< "mv "<< CPPFile::objFullName(*iter) <<"  ./";//<< CPPFile::objName(*iter);


		}
		fout_dst << endl;
	}


	

	

};

class LibWriter {


	ProjectDebugFactory projectFactory;


public:

	

	void  writeProject() {

		Project project = projectFactory .getProject(PathMoum::podolibPath());
		


		std::ofstream fout(project.getMakefileFile ().string().c_str()  );

		if(!fout.is_open()) {
			//throw std::exception("NotOpen");

			return ;

		}

		

		FileList cppList ;
		FileUtil::gather_recusive (project.getDirectory(),buildExtents("cpp"),cppList);	
		
		DependInfo dependInfo;
		{


			FileList filtedFileInDependZone ;
            FileUtil::gather_recusive(project.getDirectory(),buildExtents("cpp","h","p"),filtedFileInDependZone);
		

			dependInfo= DependInfoComputer::getDependInfo(filtedFileInDependZone);

		}


		writeHead(project,fout);
		writeOBJS(cppList,fout);
		writeTARGET(project,fout);
		writeLIBRull(project,fout);
		CommonWriter::writeClean (fout);
		writeOBJRull(dependInfo  ,cppList,project,fout); 
	
		
		



	}

	void writeHead (Project &project , std::ofstream &fout_dst) {
		CommonWriter::writeHead(project ,fout_dst);
	}

	
	void writeOBJS ( FileList &cppFiles,	std::ofstream &fout_dst) {
		
		CommonWriter::writeOBJS (cppFiles,fout_dst);
	
		
	}

	
	void writeTARGET(Project &project,std::ofstream &fout){
		fout<<endl;
		fout << "TARGET =" <<project.getLibraryFile().leaf()<<endl;

	}

	void writeLIBRull(Project & project,std::ofstream &fout_dst) {

		fout_dst<<endl;
		fout_dst<<project.getProjectName () <<" : ${OBJS}"<<endl;
		//fout_dst<<"\t rm "<<PathUtil::relatePath(PathMoum::podolibPath (),project.getMakefileDir ())<<project.getLibraryFile().leaf() <<endl ;
		fout_dst<<"\t ar rs "<<" ${TARGET} ${OBJS} " <<endl ;
		fout_dst<<endl;
	}

	void writeOBJRull ( DependInfo const & dependInfo,
							FileList &cppFiles,	Project project , std::ofstream &fout_dst) {


			CommonWriter::writeOBJRull(dependInfo,cppFiles,project,fout_dst);
	}






	
	


};





class ExeWriter{

	ProjectDebugFactory projectFactory;

public:
	void writeHead (Project &project , std::ofstream &fout_dst) {
		CommonWriter::writeHead(project ,fout_dst);
	}







	void writeOBJS ( FileList &cppFiles,	std::ofstream &fout_dst) {
		
		CommonWriter::writeOBJS (cppFiles,fout_dst);
		
	}

	void writeTARGET(std::ofstream &fout){
		fout<<endl;

		fout << "TARGET = main" <<endl;

	}

	
	void writeOBJRull ( DependInfo const & dependInfo, 
							FileList &cppFiles,	Project project , std::ofstream &fout_dst) {
		
		
			CommonWriter::writeOBJRull(dependInfo,cppFiles, project,fout_dst);
	}


	

	


	void writeExeRull (Project project,  std::ofstream &fout_dst) {

		
	
		Project projLib =  projectFactory.getProject (PathMoum::podolibPath() );
	
		fout_dst<< endl;

		

		fout_dst<<"${TARGET} : podo ${OBJS} "<<endl;
		fout_dst<<"\t"<< "${LD} ${LDFLAG} -o${TARGET} ${OBJS} "
			<<" -lX11  " << " -L"<<PathMoum::xlibPath ()
			<<" -L"<<PathUtil::relatePath(projLib.getMakefileDir().getPath() ,project.getMakefileDir().getPath()) <<	"  -lpodo  ";
		
	}
	
	
	void writePODORULL(Project project,std::ofstream &fout_dst){


		Project projLib =  projectFactory.getProject (PathMoum::podolibPath() );

		fout_dst<< endl;



		fout_dst<<"podo : "<<endl;
		fout_dst<<"\t cd "	<<PathUtil::relatePath(projLib.getMakefileDir() ,project.getMakefileDir())<<";  make;" ;

		

	}

	
	void writeClean(Project project, std::ofstream &fout_dst) {

		Project projLib =  projectFactory.getProject (PathMoum::podolibPath() );

		fout_dst <<endl;

		fout_dst <<"cleanpodo: \n";
  		fout_dst <<"\t cd "<<PathUtil::relatePath(projLib.getMakefileDir() ,project.getMakefileDir())<< ";  make -i clean;" ;
		fout_dst <<endl;

		fout_dst <<"clean : cleanpodo"<<endl;
		fout_dst <<'\t'<< "rm ${OBJS} ${TARGET}  "<< endl;
		fout_dst<<"\t";

	}






	void writeProject(LegalDirectory const & prjDir) {



		
		FileList cppFiles ; 	
		{
			
			
			FileUtil::gather_recusive(prjDir, buildExtents("cpp"), cppFiles);

		}



		

		
		DependInfo dependInfo;
		{

			path dependPath = PathMoum::podolibPath () ;
			path dependPath2 = PathMoum::podoCustomPath () ;

			FileList filtedFileInDependZone ;
			FileUtil::gather_recusive(dependPath,buildExtents("cpp","h","p"), filtedFileInDependZone);
			FileUtil::gather_recusive(dependPath2,buildExtents("cpp","h","p"), filtedFileInDependZone);
			FileUtil::gather_recusive(prjDir, buildExtents("cpp","h","p"), filtedFileInDependZone);


		
			
			dependInfo= DependInfoComputer::getDependInfo(filtedFileInDependZone);

		}

		

		Project project = projectFactory.getProject(prjDir);

		std::ofstream fout_src(project.getMakefileFile ().string().c_str() );
		

		writeHead(project,fout_src);


		writeOBJS(cppFiles,fout_src);	
		writeTARGET(fout_src);
        writeExeRull(project ,fout_src);
		writePODORULL(project,fout_src);
		writeClean (project,fout_src);
		writeOBJRull( dependInfo,cppFiles,project , fout_src);
		
					
		

	}

};




void writeExampleProjects() {

	ExeWriter exeWriter;
	
   



	boost::filesystem::path examplePath("../../example");
    
	boost::filesystem::directory_iterator iterEnd;	
	for( boost::filesystem::directory_iterator iter (examplePath) ; iter != iterEnd; ++iter) {		
		if(boost::filesystem::is_directory(*iter)) {

			if(iter->leaf() != std::string("cvs")
				&& iter->leaf() != std::string("cvS")
				&& iter->leaf() != std::string("cVs")
				&& iter->leaf() != std::string("cVS")
				&& iter->leaf() != std::string("Cvs")
				&& iter->leaf() != std::string("CvS")
				&& iter->leaf() != std::string("CVs")
				&& iter->leaf() != std::string("CVS")


			) {
				LegalDirectory prjDir( *iter);


				exeWriter.writeProject(prjDir);
			}

		}

	}

}




int main(int argc, char *argv[]) {


	

	
		try {

			LibWriter libWriter;
			libWriter.writeProject ();
	

			if(argc ==  1) {
				writeExampleProjects();
				return 0 ;
			}
			else {

				using namespace boost::filesystem;			
				ExeWriter exeWriter;
				exeWriter.writeProject(path (string(argv[1])));
				return 0;
	
			}
			

		}catch(std::exception &e) {
			std::cerr<<" \nErroeOccured Error : "<<e.what()<<"while generating Project "<<argv[1] ;
			std::cerr<<endl;
			return 0;
		}

	



	



	return 0;

}









