#ifndef _RECT_H
#define _RECT_H

#include "Rect.p"
#include "Point.h"
#include "Size.h"

#include <algorithm>
#include <iostream>

#include "Debug.h"



namespace podo{
	 
    class Rect  {
    private:
		Point _point;
		Size _size;

        public :

        Rect()  {

        }

        Rect(int32_t initX, int32_t initY, uint32_t initWidth, uint32_t initHeight) 
			: _point(initX, initY),
           _size(initWidth, initHeight) {

        }

        Rect(Point const & initPt1, Point const & initPt2) :
           _point(std::min(initPt1.getX(), initPt2.getX()), std::min(initPt1.getY(), initPt2.getY())),
           _size(1 + std::max(initPt1.getX(), initPt2.getX()) - std::min(initPt1.getX(), initPt2.getX()),
           1 + std::max(initPt1.getY(), initPt2.getY()) - std::min(initPt1.getY(), initPt2.getY())) {


        }

        Rect(Point const & initPt, Size const & initSize) : _point(initPt), _size(initSize) {
        }

        bool isIn(Point const & pt_r) const {
			if(this->isNull()){
                return false;
			}
            return (getX() <= pt_r.getX()) && (pt_r.getX() <= getX2()) && (getY() <= pt_r.getY()) && (pt_r.getY() <= getY2());


        }

        bool isIn(Rect const & rt_r) const {
			if(this->isNull()){
                return false;
			}
            return isIn(Point(rt_r.getX(), rt_r.getY())) &&
               isIn(Point(rt_r.getX() + rt_r.getWidth() - 1, rt_r.getY() + rt_r.getHeight() - 1));

        }

		int32_t const getX() const {
			return _point.getX();
		}

		int32_t const getY() const {
			return _point.getY();
		}

		uint32_t const getWidth() const {
			return _size.getWidth();
		}

		uint32_t const getHeight() const {
			return _size.getHeight();
		}

		void setX(int32_t x)  {
			return _point.setX (x);
		}

		void setY(int32_t y) {
			return _point.setY(y);
		}

		void setWidth(int32_t width)  {
			return _size.setWidth(width);
		}

		void setHeight(int32_t height){
			return _size.setHeight(height);
		}

        int32_t getX2() const {
            if (getWidth() == 0) {
                assertExp(false);
            }
            return getX() + getWidth() - 1;

        }

        int32_t getY2() const {


            if (getHeight() == 0) {
                assertExp(false);
            }

            return getY() + getHeight() - 1;

        }

		int32_t getNextX() const {            
            return getX() + getWidth() ;
        }

        int32_t getNextY() const {
            return getY() + getHeight();

        }


		Point const toPoint() const {
			return _point;

		}

		Size const toSize()  const {
			return _size;

		}

		


        Rect & operator &= (Rect const & rt_r);

        Rect operator & (Rect const & rt_r) const {
            Rect rt(* this);
            rt &= rt_r;

            return rt;

        }

        bool isNull() const {
            return getWidth() == 0 || getHeight() == 0;


        }



        Rect & addPoint(Point const & pt_r) {
            this->setX(this->getX() + pt_r.getX());
            this->setY(this->getY() + pt_r.getY());

            return * this;

        }

        Rect & subPoint(Point const & pt_r) {
            this->setX(this->getX() - pt_r.getX());
            this->setY(this->getY() - pt_r.getY());

            return * this;
        }

        bool operator == (Rect const & rt) const {
            return (getX() == rt.getX()) && (getY() == rt.getY()) && (getWidth() == rt.getWidth()) &&
               (getHeight() == rt.getHeight());


        }

        bool operator != (Rect const & rt) const {
            return !operator == (rt);
        }

        virtual String toString() const;

        friend std::ostream & operator << (std::ostream & os, Rect const & rt) {

            os << "Rect( " << rt.getX() << "," << rt.getY() << "," << rt.getWidth() << "," << rt.getHeight() << ")";

            return os;


        }

    };

	


}

 //namespace podo

#endif
//RECT_H

