#ifndef PODP_CLIPBOARD_H_
#define PODP_CLIPBOARD_H_

#include <boost/shared_ptr.hpp>
#include "SystemApi.h"
#include "../global/Debug.h"
#include "../global/String.h"


namespace podo {


	
	

	class Clipboard {



#if defined(_PODO_POSIX)


#elif defined(_PODO_WIN32)
		static const int ALLOC_FLAG = GHND | GMEM_SHARE;
#endif






	public:

		//just For Get
		struct TextData {

		

			String _str;




#if defined(_PODO_POSIX)
			TextData() {
			}


#elif defined(_PODO_WIN32)
			TextData(HANDLE handle ){


				assertExp( GlobalSize( handle) );

				size_t _size = GlobalSize( handle) - 1 ;
				char * _data = (char*) ::GlobalLock (handle);

				_str = String(_data , _data + ( _size));
				::GlobalUnlock(handle);

			}


#endif			

			~TextData() {
				
			}			

			std::size_t size() {
				return _str.size();
			}			

			String::iterator  begin() {
				return _str.begin();
			}

			String::iterator  end() {
				return _str.end();
			}



			
			
		};

		

	public:
		//typedef UINT DataType ;

		friend class ClipboardOwner ;
		friend class ClipboardRequestor ;


	private:

	

		void openClipboard() {





#if defined(_PODO_POSIX)


#elif defined(_PODO_WIN32)

			if( ::OpenClipboard(NULL)) {
			}else {
				assertExp(false);
			}


#endif




			
		}

		void closeClipboard() {


#if defined(_PODO_POSIX)


#elif defined(_PODO_WIN32)
			if(::CloseClipboard() > 0) {
			}else {
				int ddd = ::GetLastError();
				assertExp(false);
			}

#endif


		
		}

		void clearClipboard() {


#if defined(_PODO_POSIX)


#elif defined(_PODO_WIN32)
			if(::EmptyClipboard() > 0){

			}else {
				assertExp(false);
			}


#endif

			
		}

		bool const hasTextData() {

#if defined(_PODO_POSIX)
			return false;

#elif defined(_PODO_WIN32)
			return IsClipboardFormatAvailable( CF_TEXT ) > 0;

#endif
			
		}		
		


		TextData  getTextData() {

#if defined(_PODO_POSIX)
			return TextData();

#elif defined(_PODO_WIN32)
			assertExp(hasTextData());
			HANDLE handle = ::GetClipboardData (CF_TEXT);
			if(handle) {
			}
			else {
				int ddd = ::GetLastError();
				assertExp(false);
			}
			return TextData(handle);


#endif

		}
		
		template <typename Iterator>
		void setTextData(Iterator begin , Iterator end ){



#if defined(_PODO_POSIX)

#elif defined(_PODO_WIN32)

			String toInput;

			if(2 ==sizeof(*begin)) {
				WString wstr(begin, end);
				String str = toString (wstr);
				toInput = toCariageString(str);
			}else {
				String str  (begin, end);
				toInput = toCariageString(str);
			}

			

			HANDLE _handle = ::GlobalAlloc( ALLOC_FLAG ,toInput.size() + 1 );


			char * _data =(char*) ::GlobalLock(_handle);
			std::copy(toInput.begin(),toInput.end() ,_data);
			_data[toInput.size()  ] = '\0';

			::GlobalUnlock(_handle);
			/*if(::GlobalUnlock(_handle)> 0) {

			}else {
				int ddd = GetLastError();
				assertExp(false);

			}*/


			::SetClipboardData (CF_TEXT,_handle);



#endif




		}


		static Clipboard& instance() {

			static Clipboard _instance;
			return _instance;
		}
		
		
	};

	class ClipboardOwner {
	public:

		ClipboardOwner () {
			Clipboard::instance().openClipboard() ;
			Clipboard::instance().clearClipboard();
		}

		~ClipboardOwner () {
			Clipboard::instance().closeClipboard() ;
		}

		template <typename Iterator>
		void setTextData(Iterator begin , Iterator end ){
			Clipboard::instance().setTextData(begin,end);
		}


	};

	class ClipboardRequestor {
	public:

		ClipboardRequestor () {
			Clipboard::instance().openClipboard() ;			
		}

		~ClipboardRequestor () {
			Clipboard::instance().closeClipboard() ;
		}

		Clipboard::TextData getTextData() {
			return Clipboard::instance().getTextData ();			
		}		

		bool const hasTextData() {
			return Clipboard::instance().hasTextData();

		}

	};

}



#endif
//PODP_CLIPBOARD_H_

