#ifndef FILTERING_H
#define FILTERING_H

#include "../global/Line.h"
#include "../global/Rect.h"
#include <set>

namespace podo{
	class Filter {

	private:
		
		static int32_t lowerbound (int32_t from1, int32_t to1, int32_t from2, int32_t to2) {

			int32_t const min1 = std::min(from1, to1);
			if (std::min(from2, to2) <= min1 && min1 <= std::max(from2, to2)) {
				return min1;
			}

			int32_t const min2 = std::min(from2, to2);
			if (std::min(from1, to1) <= min2 && min2 <= std::max(from1, to1)) {
				return min2;
			}


			assertExp(false);
			return 0;

		}

		static int32_t upperbound (int32_t from1, int32_t to1, int32_t from2, int32_t to2) {

			int32_t const max1 = std::max(from1, to1);
			if (std::min(from2, to2) <= max1 && max1 <= std::max(from2, to2)) {

				return max1;

			}

			int32_t const max2 = std::max(from2, to2);
			if (std::min(from1, to1) <= max2 && max2 <= std::max(from1, to1)) {
				return max2;
			}


			assertExp(false);
			return 0;



		}

		static bool const hasUnionRegion (int from1, int to1, int from2, int to2) {

			int min1 = std::min(from1, to1);
			if (std::min(from2, to2) <= min1 && min1 <= std::max(from2, to2)) {
				return true;
			}

			int max1 = std::max(from2, to2);
			if (std::min(from1, to1) <= max1 && max1 <= std::max(from1, to1)) {
				return true;
			}

			return false;

		}

		static bool const isVertical(Point const from, Point const to) {
			return  from.getX()  == to.getX()    ;

		}

		static bool const isHorizontal(Point const from, Point const to) {
			return  from.getY()  == to.getY()    ;
		}


		


		static int32_t const computeYOnVertical(int32_t const  verticalX, Point const from, Point const to) {

			int32_t result = (verticalX - from.getX()) * (to.getY() - from.getY()) /
				(to.getX() - from.getX()) + from.getY();

			return result;
		}

		static int32_t const computeXOnHorizontal(int32_t const  horizontalY, Point const from, Point const to) {

			int32_t result = (horizontalY - from.getY()) * (to.getX() - from.getX()) /
				(to.getY() - from.getY()) + from.getX();


			return result;
		}

	

		static Point const computePointOnSouth(Rect const & region, Point const from	,Point const to	){
			Point southPt(computeXOnHorizontal(region.getY2(),from,to), region.getY2());
			return southPt ;		
		}

		static Point const computePointOnNorth(Rect const & region, Point const from	,Point const to	){
			Point northPt(computeXOnHorizontal(region.getY(),from,to), region.getY());
			return northPt;
		}

		static Point const computePointOnEast(Rect const & region, Point const from	,Point const to	){
			Point eastPt(region.getX2(), computeYOnVertical(region.getX2(),from,to));
			return eastPt ;
		}

		static Point const computePointOnWest(Rect const & region, Point const from	,Point const to	){
			Point westPt(region.getX(), computeYOnVertical(region.getX(),from,to));
			return westPt;
		}
		


		static Line const computeAtVertical(Rect const &region, Point const from, Point const to) {
			// ϶  ׷  ִ Լ ̴.

	
			if (hasUnionRegion(from.getY(), to.getY(), region.getY(), region.getY2())) {
				Point newFrom;
				Point newTo;
				newFrom = Point(from.getX(),
				lowerbound(from.getY(), to.getY(), region.getY(), region.getY2()));
				newTo = Point(from.getX(),
				upperbound(from.getY(), to.getY(), region.getY(), region.getY2()));
				
				return Line(true,newFrom,newTo);

			} else {
				return Line::nullLine ;
			}


		}

		static Line const computeAtHorizontal(Rect const &region,Point const from, Point const to) {
			// ϶  ׷  ִ Լ ̴.
		
	
			if (hasUnionRegion(from.getX(), to.getX(), region.getX(), region.getX2())) {
				Point newFrom;
				Point newTo;
				newFrom = Point(lowerbound(from.getX(), to.getX(), region.getX(), region.getX2()), from.getY());
				newTo = Point(upperbound(from.getX(), to.getX(), region.getX(), region.getX2()), from.getY());
				return Line(true,newFrom,newTo);

			} else {
				return Line::nullLine ;			
			}
		}

		static bool const isRegionIn(Point const reg1,Point const reg2,Point const pointFor) {
			int32_t miX = std::min(reg1.getX (),reg2.getX());
			int32_t maX = std::max(reg1.getX (),reg2.getX());
			int32_t miY = std::min(reg1.getY (),reg2.getY());
			int32_t maY = std::min(reg1.getY (),reg2.getY());

			return miX <= reg1.getX () &&
				reg1.getX () <= maX &&
				miY <= reg2.getY () &&
				reg2.getY () <= maY ;



		}

		static Line const computeAtUnKnown(Rect const &region, Point const from, Point const to) {
			typedef std::set < Point, Point::PointLess > Points;	
			Points temp;


			Point pointSouth =  computePointOnSouth(region,from,to);
			Point pointNorth =  computePointOnNorth(region,from,to);
			Point pointEast =  computePointOnEast(region,from,to);
			Point pointWest =  computePointOnWest(region,from,to);
	           
			if(region.isIn(pointSouth ) && isRegionIn(from,to,pointSouth)) {
				temp.insert(pointSouth); 
			}		
              
			if(region.isIn(pointNorth ) && isRegionIn(from,to,pointNorth)) {
				temp.insert(pointNorth); 
			}		
            
			if(region.isIn(pointEast ) && isRegionIn(from,to,pointEast)) {
				temp.insert(pointEast); 
			}
			
			if(region.isIn(pointWest ) && isRegionIn(from,to,pointWest)) {
				temp.insert(pointWest); 
			}
			
			if (region.isIn(from)) {
				temp.insert(from);
			}
			if (region.isIn(to)) {
				temp.insert(to);
			}



			if (temp.size() == 2) {

				Points::iterator iter;
				iter = temp.begin();

				Point newFrom = * iter;
				++iter;
				Point newTo = * iter;

				return Line(true,newFrom,newTo);
			} else if (temp.size() == 1) {
				Points::iterator iter;
				iter = temp.begin();

				Point newFrom = * iter;
				Point newTo = * iter;

				return Line(true,newFrom,newTo);

			}
			else {
				assertExp(temp.size() == 0);
				return Line::nullLine ;
			}

			
		}


	public:            

				static Line filtLine(Rect const & region, Point const from	,Point const to);

				
	};
}

#endif
//FILTERING_H

