#ifndef PODO_FONTHOLDERS_H_
#define PODO_FONTHOLDERS_H_


#include "SystemApi.h"
#include "posix/GlobalHandle.h"
#include "../global/Basic.h"
#include "../global/Font.h"

namespace podo {

	class FontHolderS {


#if defined(_PODO_POSIX)
		typedef ::Font FontHandle ;
#elif defined(_PODO_WIN32)
		typedef ::HFONT FontHandle;
#endif



	public:


		FontHandle _fontHangul;
		FontHandle _fontEnglish;

		
		podo::Font _font;

		FontHolderS(podo::Font const &font):_font(font){

#if defined(_PODO_POSIX)
			_fontHangul = XLoadFont(getGlobalHandle(), "fixed");
			_fontEnglish = XLoadFont(getGlobalHandle(), "fixed");
#elif defined(_PODO_WIN32)
			_fontHangul = ::CreateFont(_font.getHeight(),0,0,0,0,0,0,0,HANGEUL_CHARSET,3,2,1,VARIABLE_PITCH | FF_ROMAN,"Gulim");
			_fontEnglish = ::CreateFont(_font.getHeight(),0,0,0,0,0,0,0,ANSI_CHARSET,3,2,1,VARIABLE_PITCH | FF_ROMAN,"Times New Roman");
#endif

		}

		~FontHolderS(){

#if defined(_PODO_POSIX)
		XUnloadFont(getGlobalHandle(), _fontHangul);
		XUnloadFont(getGlobalHandle(), _fontEnglish);

#elif defined(_PODO_WIN32)
			::DeleteObject(_fontHangul);
			::DeleteObject(_fontEnglish);		
#endif		




		}


		uint32_t getWidth(wchar_t const ch) ;

		template<typename Iterator>
		uint32_t getWidth(Iterator const & begin, Iterator const & end) {


			uint32_t result = 0;



			for( Iterator iter = begin ; iter != end ; ++ iter ) {

				wchar_t ch = *iter ;

				uint32_t temp = this->getWidth(ch);

                result += temp;
				//trace("$"<<result<<"^"<<temp<<"&"<<(int)ch);

			}

			return result;

		}





		uint32_t getHeight(wchar_t ch) {

			return  _font.getHeight();
		}


		FontHandle &getFontHandle(wchar_t ch) {

			if( 0xAC00<= ch && ch <= 0xD7AF){//isKorean
				return _fontHangul;
			}else {
				return _fontEnglish;
			}
		}		



	};
};

#endif
// /
///PODO_FONTHOLDERS_H_





