
#include "InOutDrawStructDefault.h"
#include "../global/Rect.h"


#include "../global/Brush.h"
#include "../global/Pen.h"
#include "../global/Font.h"


#include "../global/Line.h"
#include "FontHolderS.h"
#include "Filter.h"

#include "posix/GlobalHandle.h"

#include <map>

namespace podo{






		#if defined(_PODO_WIN32)


		int ColorToRGB(Color const & color) {

		
				
			int rgb = RGB(color.getRed().toInt(),
				color.getGreen().toInt() ,
				color.getBlue().toInt());

			return rgb;


		}

			

		#elif defined(_PODO_POSIX)









		int ColorToRGB(Color const & color) {

			static std::map < uint32_t, int32_t > colors;

			#define RGB(r,g,b)	 ((r<<0) + (g<<8) + (b<<16))

			const uint32_t rgbs
				=	RGB( color.getRed().toInt() ,  color.getGreen().toInt() ,  color.getBlue().toInt() );


			if (colors.find(rgbs) == colors.end()) {
				Colormap cmap;
				XColor co;

				assertExp(getGlobalHandle() != 0);
				cmap = DefaultColormap(getGlobalHandle(), DefaultScreen(getGlobalHandle()));


				co.red =  color.getRed().toInt() * 257;
				co.green =  color.getGreen().toInt() * 257;
				co.blue =  color.getBlue().toInt() * 257;
				int result = XAllocColor(getGlobalHandle(), cmap, & co);

				assertExp(result != 0);

				//printf("Color Created (%d,%d,%d) As (%d ,%d, %d ,: %d, )",
				//	r.toInt(), g.toInt(), b.toInt(), co.red, co.green, co.blue, co.pixel);


				colors.insert(std::pair<uint32_t,int32_t>(rgbs,co.pixel) );

				//colors[rgbs] = co.pixel;

			}

			return colors[rgbs];

		}


		#endif







		


		void InOutDrawStructDefault::fillRect	(InOutDrawStructDefault::DrawableHandle drawableHandle,Rect const & region, Point const &startPoint ,
			Brush const &brush , Rect const & rt) {




			Rect tempRect = rt;
			Rect const rectOnClient = tempRect.addPoint(startPoint);
			


			Rect smallRect =  region & rectOnClient;

			if (smallRect.isNull()) {
				return ;
			}


		#if defined(_PODO_WIN32)
			
			int rgb = RGB(brush.getColor().getRed().toInt(),
				brush.getColor().getGreen().toInt() ,
				brush.getColor().getBlue().toInt());

			HBRUSH newBrush =::CreateSolidBrush(rgb);
			if (newBrush == NULL) {
				//		DWORD aaa = ::GetLastError();
					assertExp(false);
			}



			RECT temprt;
			temprt.left = smallRect.getX();
			temprt.right = smallRect.getX2()+1;
			temprt.top = smallRect.getY();
			temprt.bottom = smallRect.getY2()+1;

			if (NULL ==::FillRect(drawableHandle, & temprt, newBrush)) {
				assertExp(false);
			}


			::DeleteObject(newBrush);


		#elif defined(_PODO_POSIX)

			XSetForeground(getGlobalHandle(), drawableHandle.gc, ColorToRGB(brush.getColor()));

			XFillRectangle(getGlobalHandle(), drawableHandle.drawable, drawableHandle.gc,
				smallRect.getX(), smallRect.getY(),
				smallRect.getWidth(),	smallRect.getHeight());



		#endif

			
		}


		void InOutDrawStructDefault::lineLine	(InOutDrawStructDefault::DrawableHandle drawableHandle,Rect const & region, Point const &startPoint ,
			Pen const &pen ,  Point const & from, Point const & to) {


			Point fromOnTop = from + startPoint ;
			Point toOnTop = to + startPoint ;
		

			Line line = Filter::filtLine(region,fromOnTop,toOnTop);

			

		#if defined(_PODO_WIN32)


			int rgb = ColorToRGB(pen.getColor());

	    
	    
			HPEN tempPen = CreatePen(PS_SOLID, 1,rgb );
			HPEN oldPen = (HPEN)::SelectObject(drawableHandle , tempPen);
	    
	    

			MoveToEx(drawableHandle, line.getPt1().getX(), line.getPt1().getY(), (LPPOINT)NULL);
			LineTo(drawableHandle, line.getPt2().getX(), line.getPt2().getY());

	    
			(HPEN)::SelectObject(drawableHandle, oldPen);
			::DeleteObject(tempPen);
	    
		#elif defined(_PODO_POSIX)
	    
		/*printf("Region:(%d,%d ,%d ,%d ,)lineLine (%d,%d) (%d,%d)\n",
    			clippingRegion.getX(),clippingRegion.getY(),clippingRegion.getWidth(),clippingRegion.getHeight(),
    			newFrom.getX(),newFrom.getY(),newTo.getX(),newTo.getY());*/
	    
			XSetForeground(getGlobalHandle(), drawableHandle.gc, ColorToRGB(pen.getColor()));
	    
			XDrawLine(getGlobalHandle(), drawableHandle.drawable, drawableHandle.gc, line.getPt1().getX(),
				line.getPt1().getY(),line.getPt2().getX(), line.getPt2().getY());
	    

		#endif
    		 
        }

		
		void writeText (InOutDrawStructDefault::DrawableHandle drawableHandle,Rect const & regionOnClient, 
				Font const &font, WString const &text, 	Point const & pointOnClient	 ) 	{




		}

		Point getMovePoint(Size const &fontSize, Font::VerticalWriteMode verticalMode ,	Font::HorizontalWriteMode horizontalMode ) {

			//     ϴ ǥ..

			int x = 0 ;


			if(horizontalMode == Font::H_LEFT  ) {
				x = x - 0;
			}else if(horizontalMode == Font::H_CENTER ){
				x = x - int(fontSize.getWidth() /2 );

			}else if(horizontalMode == Font::H_RIGHT ) {
				x =  x - int(fontSize.getWidth()) ;

			}else {
				assertExp(false);
			}

			int y = 0;
			if(verticalMode == Font::V_TOP) {
				y = y - 0;
			}else if(verticalMode == Font::V_CENTER  ){
				y = y - int(fontSize.getHeight() /2) ;

			}else if(verticalMode == Font::V_BOTTOM ) {
				y = y - int(fontSize.getHeight() ) ;
			}else {
				assertExp(false);
			}

			Point result (x,y);

			return result;





		}





		void InOutDrawStructDefault::write	(DrawableHandle drawableHandle,Rect const & regionOnClient,
				Point const &startPointOnClient , Font const &font, WString const &text,
				Point const & pointOnComponent,
				Font::VerticalWriteMode verticalMode ,
				Font::HorizontalWriteMode horizontalMode ) 	{






			FontHolderS holder(font);


			Size const fontSize(holder.getWidth(text.begin(), text.end() ), font.getHeight());

			//trace("\n fontSize :" << fontSize.toString() <<"textSize : " << text.size() );
			Point fontPointOnClient = startPointOnClient + pointOnComponent + getMovePoint(fontSize,verticalMode,horizontalMode);
			//trace ("movePoint : "<< getMovePoint(fontSize,verticalMode,horizontalMode).toString( )  );
			Rect const smallRectOnClient = regionOnClient & Rect(fontPointOnClient,fontSize);


			Point const pointOnSmallRect(fontPointOnClient.getX() - smallRectOnClient.getX()   ,fontPointOnClient.getY() - smallRectOnClient.getY()  );

		//	trace("  \n smallRet : "<<smallRectOnClient.toString() <<"pointOnSmallRect :" <<pointOnSmallRect.toString() <<"fontPoint : "<< fontPointOnClient.toString());


		#if defined(_PODO_WIN32)


				HDC tempDc = CreateCompatibleDC(drawableHandle);
				HBITMAP bitmap =
					CreateCompatibleBitmap(drawableHandle,
					smallRectOnClient.getWidth (),smallRectOnClient.getHeight() );

				HBITMAP oldBitmap = (HBITMAP)::SelectObject(tempDc,bitmap);







					::BitBlt(tempDc,
						0,0,smallRectOnClient.getWidth(),smallRectOnClient.getHeight(),
						drawableHandle,
						smallRectOnClient.getX(),smallRectOnClient.getY(),
						SRCCOPY);

					::SetTextColor( tempDc,ColorToRGB( font.getColor()));
					::SetTextAlign( tempDc,TA_LEFT | TA_BOTTOM);


					int currentTextWidth = 0;
					
					for(size_t index = 0; index < text.size() ; ++index) {

						HFONT oldFont = (HFONT) ::SelectObject(tempDc,holder.getFontHandle(text[index]) );
						::TextOutW(tempDc,pointOnSmallRect.getX() + currentTextWidth ,
							pointOnSmallRect.getY() + font.getHeight()  , &text[index] , 1  );
						::SelectObject(tempDc,oldFont);

						currentTextWidth += holder.getWidth(text[index]);
					}




					::BitBlt(drawableHandle,
						smallRectOnClient.getX(),smallRectOnClient.getY(),
						smallRectOnClient.getWidth(),smallRectOnClient.getHeight(),
						tempDc,0,0,SRCCOPY);


				::SelectObject(tempDc,oldBitmap);
				::DeleteObject(bitmap);
				::DeleteDC(tempDc);



		#elif defined(_PODO_POSIX)

		if(smallRectOnClient.isNull()) {
			return;
		}


		//trace("\n CreatePixmap "<<smallRectOnClient.getWidth()<<"  "<<smallRectOnClient.getHeight()<<"  "<<DefaultDepthOfScreen(getDefaultScreen() ));


		Pixmap virtualPixmap = XCreatePixmap(getGlobalHandle() , drawableHandle.drawable,
			smallRectOnClient.getWidth (),smallRectOnClient.getHeight(),
			DefaultDepthOfScreen(DefaultScreenOfDisplay(getGlobalHandle()))
			);


		GC virtualGC = XCreateGC(getGlobalHandle(), virtualPixmap,0,0);





		XCopyArea(getGlobalHandle(), drawableHandle.drawable, virtualPixmap, virtualGC
			, smallRectOnClient.getX(),smallRectOnClient.getY(), smallRectOnClient.getWidth(),smallRectOnClient.getHeight(),
			0, 0);




		XSetForeground(getGlobalHandle(), drawableHandle.gc, ColorToRGB(font.getColor()));


		int currentTextWidth = 0;

		for(size_t i = 0; i < text.size(); ++i) {


			char buf[3] = {0,};
			wchar_t ch = text[i];

			wcrtomb(buf,ch,NULL);

			XSetFont(getGlobalHandle(), virtualGC,holder.getFontHandle(ch) );


			XDrawString(getGlobalHandle(),	virtualPixmap,virtualGC,
				pointOnSmallRect.getX() + currentTextWidth, pointOnSmallRect.getY() + font.getHeight() ,

				buf,1);

			currentTextWidth += holder.getWidth(ch);
		}





		XCopyArea(getGlobalHandle(),virtualPixmap,  drawableHandle.drawable, drawableHandle.gc,
		  0,0, smallRectOnClient.getWidth(),smallRectOnClient.getHeight(), smallRectOnClient.getX(),smallRectOnClient.getY());







		XFreeGC(getGlobalHandle(),virtualGC);
		XFreePixmap(getGlobalHandle() ,virtualPixmap);






		#endif






			/*
			ch = '';



			wcrtomb(buf,ch,NULL);


			if(buf[0] & 0x80) {  //if Hangul

				trace("\n  WChar is  " <<(int) ch<<" Printing Hangul  :  "
					<< (int)buf[0]<<","<< (int)buf[1]<<","<< (int)buf[2] );



				::Font temp = XLoadFont(getGlobalHandle(), "fixed");
				XSetFont(getGlobalHandle(), drawableHandle.gc,temp);
				XDrawString(getGlobalHandle(),	drawableHandle.drawable,drawableHandle.gc,
					rectOnClient.getX() + i * 10,rectOnClient.getY() + 14 ,
					buf, 1);
				XUnloadFont(getGlobalHandle(), temp);

			}
			else {

			*/
//				trace("\n  WChar  is :  " <<(int) ch<<" Printing English  :  "
//					<< (int)buf[0]<<","<< (int)buf[1]<<","<< (int)buf[2] );







				


		}
		//void InOutDrawStructDefault::write		(
		//	InOutDrawStructDefault::DrawableHandle drawableHandle,
		//	Rect const & region, Point const &startPoint ,
		//	Font const &font, WString const &text, Rect const & rect,
		//	Font::VerticalWriteMode verticalMode ,
		//	Font::HorizontalWriteMode horizontalMode  		) {



		//	Rect temp = rect;
		//	Rect const rectOnClient = temp.addPoint(startPoint);


		//	Rect smallRectOnClient = rectOnClient & region;
		//	if(smallRectOnClient.isNull()) {
		//		return;
		//	}
		//		
		//#if defined(_PODO_WIN32)


		//	if(smallRectOnClient == rectOnClient) {

		//		RECT rt;
		//		rt.left = rectOnClient.getX();
		//		rt.right = rectOnClient.getX2()+1;
		//		rt.top = rectOnClient.getY();
		//		rt.bottom = rectOnClient.getY2()+1;




		//		::SetTextColor(drawableHandle,ColorToRGB(font.getColor()));

		//		::DrawTextW (drawableHandle,text.c_str(),static_cast<int>(text.size()),&rt,DT_CENTER | DT_VCENTER | DT_SINGLELINE);

		//	} else {
		//		//ڰ 䱸  ׸ ִ 
		//		//Ѿ ó Ѵ..
		//		// ۸ ̿ Ѵ..


		//		HDC tempDc = CreateCompatibleDC(drawableHandle);
		//		HBITMAP bitmap = 
		//			CreateCompatibleBitmap(drawableHandle,
		//			rectOnClient.getWidth (),rectOnClient.getHeight() );

		//		HBITMAP oldBitmap = (HBITMAP)::SelectObject(tempDc,bitmap);



		//		
		//			RECT rt;
		//			rt.left = 0;
		//			rt.right = rectOnClient.getWidth();
		//			rt.top = 0;
		//			rt.bottom = rectOnClient.getHeight();


		//			::BitBlt(tempDc,
		//				0,0,smallRectOnClient.getWidth(),smallRectOnClient.getHeight(),
		//				drawableHandle,
		//				smallRectOnClient.getX(),smallRectOnClient.getY(),	
		//				SRCCOPY);


		//			::SetTextColor(tempDc,ColorToRGB( font.getColor()));					
		//			::DrawTextW(tempDc,	text.c_str(),static_cast<int>(text.size()),&rt,
		//				DT_CENTER | DT_VCENTER | DT_SINGLELINE);


		//			

		//			::BitBlt(drawableHandle,
		//				smallRectOnClient.getX(),smallRectOnClient.getY(),
		//				smallRectOnClient.getWidth(),smallRectOnClient.getHeight(),
		//				tempDc,0,0,SRCCOPY);


		//		::SelectObject(tempDc,oldBitmap);
		//		::DeleteObject(bitmap);				
		//		::DeleteDC(tempDc);


		//	}

		//#elif defined(_PODO_POSIX)


		//

		//XSetForeground(getGlobalHandle(), drawableHandle.gc, ColorToRGB(font.getColor()));

		//char buf[10];		


		//for(size_t i = 0; i < text.size(); ++i) {
		//	wchar_t ch = text[i];
		//	
		//	wcrtomb(buf,ch,NULL);
		//	
		//	if(buf[0] & 0x80) {
		//		
		//		
		//	}
		//	else {

		//		::Font temp = XLoadFont(getGlobalHandle(), "fixed");
		//		XSetFont(getGlobalHandle(), drawableHandle.gc,temp);
		//		XDrawString(getGlobalHandle(),drawableHandle.drawable,drawableHandle.gc,rectOnClient.getX() + i * 10,
		//			rectOnClient.getY() + 15 ,buf, 1);
		//		XUnloadFont(getGlobalHandle(), temp);
		//		
		//	}
		//}



		//	//XRectFillerangle(getGlobalHandle(), drawableHandle.drawable, drawableHandle.gc,
		//	//	rectOnClient.getX(), rectOnClient.getY(), rectOnClient.getWidth(),	rectOnClient.getHeight());


		//#endif

		//}




}
