
#ifndef PODO_SYSTEM_MESSAGERECEIVER_H_
#define PODO_SYSTEM_MESSAGERECEIVER_H_

#include "../global/Point.p"
#include "../global/Size.p"
#include "../global/MouseButton.p"
#include "../global/KeyboardKey.p"
#include "PaintDrawStruct.p"
#include "SystemApi.h"
#include "Message.p"



#include "../wrapper/listener/FocusListener.h"
#include "../wrapper/listener/SizeListener.h"

namespace podo {



	
	class MessageReceiver :public FocusListenerHaver ,public SizeListenerHaver{



		
	public:

		enum  ProcessState{
			Processed,
			NotProcessed
			
		};

		virtual ProcessState distributeMouseButtonMessage(bool const & bDowned, 
			MouseButton const& button,Point const & pointOnClient,bool bIn) = 0;
		virtual ProcessState distributeKeyMessage(bool const & bDowned, KeyboardKey  const& key) = 0;	    
		virtual ProcessState distributePaintMessage(PaintDrawStruct & drawStruct) = 0;
		virtual ProcessState distributeMouseMoveMessage(Point const & pointOnClient ,bool bIn) = 0;
		virtual ProcessState distributeMouseEnteredMessage() = 0;
		virtual ProcessState distributeMouseExitedMessage() = 0;		
		virtual ProcessState distributeCloseMessage() = 0;
		
		virtual void distributeEnforceDestroyMessage() = 0;
		

	};

	class NullMessageReceiver:public MessageReceiver {

	public:
		static MessageReceiver &instance() {
			static NullMessageReceiver ins;
			return ins;

		}



	private:
		NullMessageReceiver() {

		}

		virtual ProcessState distributeMouseButtonMessage(bool const & bDowned, 
			MouseButton const& button,Point const & pointOnClient,bool bIn) {	return NotProcessed ;}
		virtual ProcessState distributeKeyMessage(bool const & bDowned, KeyboardKey  const& key) {	return NotProcessed ;	}
		virtual ProcessState distributePaintMessage(PaintDrawStruct & drawStruct) {	return NotProcessed ;	}
		virtual ProcessState distributeMouseMoveMessage(Point const & pointOnClient,bool bIn) {	return NotProcessed ;	}
		virtual ProcessState distributeMouseEnteredMessage() {return NotProcessed ;}
		virtual ProcessState distributeMouseExitedMessage() {return NotProcessed ;}		
		virtual ProcessState distributeCloseMessage() {return NotProcessed ;}		
		virtual void distributeEnforceDestroyMessage(){	}

	};


}

#endif 
//PODO_SYSTEM_MESSAGERECEIVER_H_



