
#ifndef BOARDLAYOUTMANAGER_H
#define BOARDLAYOUTMANAGER_H

#include "LayoutManager.h"
#include "../global/Rect.h"
#include "../global/AggregationAttribute.h"
#include "Component.h"
#include "Component.h"
#include "../global/Point.h"


#include "ComponentRegionUtil.h"
#include "painter/InOutGraphics.h"
#include "../system/InOutDrawStruct.h"






namespace podo{

    
	template <int BORDER_WIDTH = 20>
	class BorderLayoutManager : public LayoutManager {
			typedef LayoutManager super;


		public:


		
			static const int BORDERWIDTH  = BORDER_WIDTH;
			static const unsigned int BORDER_COUNT = 9;

			enum Direction {
				NORTH = 0, NORTH_EAST = 1, EAST = 2, SOUTH_EAST = 3, SOUTH = 4, SOUTH_WEST = 5, WEST = 6,
				NORTH_WEST = 7, CENTER = 8
			};

		private:
			

		
		private:

			AggregationAttribute<ValueType>  datas[BORDER_COUNT];
		
		public:

			BorderLayoutManager(){


				
				

			}

			virtual void init(Container & container) {
				super::init(container);
				addComponent(Component::createTemp(),CENTER);
				
			}
			~BorderLayoutManager(){

				for (size_t index  = 0 ; index < BORDER_COUNT; ++index ) {
					if(hasComponent(  Direction(index)) ){
						removeComponent(Direction(index));
					}
				}

			}


			void addComponent(Component * component_p, Direction dir) {
	           ComponentPtr temp(component_p);
	           addComponent(temp, dir);
	       }

			void setComponent(Component * component_p, Direction dir) {
	           ComponentPtr temp(component_p);
	           setComponent(temp, dir);
	       }

	       void addComponent(ComponentPtr component_p, Direction dir)  {		  



				datas[dir].set( new ValueType( component_p, Rect() )  );
				super::registComponent(*component_p);
			}
		    void setComponent(ComponentPtr component_p, Direction dir)  {		  

				if(hasComponent(dir)) {
					removeComponent(dir);

					
				}else {

				}

				datas[dir].set( new ValueType( component_p, Rect() )  );
				super::registComponent(*component_p);
			}
		   bool const hasComponent(Direction const & dir) const {
               return datas[dir].has();
		   }

		    void  removeComponent( Direction const & dir) {
				if(hasComponent(dir)) {
					super::unregistComponent(datas[dir]->accessComponent());	  
					datas[dir].remove();

				}
				
			}

		   ComponentPtr shareComponent(Direction const & dir) {

			   return datas[dir]->shareComponent();

		   }

		  






		   virtual Rect const getChildRect(Component const &component ) const{

				
				for (size_t index  = 0 ; index < BORDER_COUNT; ++index ) {
					if(hasComponent(  Direction(index)) ){
						if(&component == &(datas[index]->accessComponent ()))	 {                    
							return datas[index]->getRect();
						}
					}
				}

				assertExp(false);

				return Rect(); 
			
			}
		   virtual Component  & findComponentAtChild(Point const & point)   {

				for (size_t index  = 0 ; index < BORDER_COUNT; ++index ) {
					if(hasComponent(  Direction(index)) ){
						Rect compRect = datas[index]->getRect ();
						if (compRect.isIn(point)) {

							Point pt = compRect.toPoint();
							Point pointOnChild = point - pt;
							return datas[index]->accessComponent().findComponent(pointOnChild);

						}
					}


				}

				return *Component::createNull();
				

			}
		   virtual void resiteChild(Size const &size) {

		
				int const NORTH_HEIGHT = (hasComponent(NORTH) || hasComponent(NORTH_WEST) ||
				hasComponent(NORTH_EAST)) ? BORDER_WIDTH : 0;


				int const SOUTH_HEIGHT = (hasComponent(SOUTH) || hasComponent(SOUTH_WEST) ||
				hasComponent(SOUTH_EAST)) ? BORDER_WIDTH : 0;


				int const WEST_WIDTH = (hasComponent(WEST) || hasComponent(NORTH_WEST) || hasComponent(SOUTH_WEST)) ?
				BORDER_WIDTH : 0;

				int const EAST_WIDTH = (hasComponent(EAST) || hasComponent(SOUTH_EAST) ||
				hasComponent(NORTH_EAST)) ? BORDER_WIDTH : 0;

				Rect const center = Rect(WEST_WIDTH, NORTH_HEIGHT, std::max < int > (0, size.getWidth() - EAST_WIDTH - WEST_WIDTH),
				std::max < int > (0, size.getHeight() - NORTH_HEIGHT - SOUTH_HEIGHT));

		        


				safeResite(CENTER, center);

				safeResite(EAST, Rect(center.getNextX() , center.getY(), EAST_WIDTH, center.getHeight()));
				safeResite(WEST, Rect(0, center.getY(), WEST_WIDTH, center.getHeight()));

				safeResite(SOUTH, Rect(center.getX(), center.getNextY() , center.getWidth(), SOUTH_HEIGHT));
				safeResite(NORTH, Rect(center.getX(), 0, center.getWidth(), NORTH_HEIGHT));

				safeResite(NORTH_EAST, Rect(center.getNextX() , 0, EAST_WIDTH, NORTH_HEIGHT));
				safeResite(NORTH_WEST, Rect(0, 0, WEST_WIDTH, NORTH_HEIGHT));

				safeResite(SOUTH_EAST, Rect(center.getNextX() , center.getNextY() , EAST_WIDTH, SOUTH_HEIGHT));
				safeResite(SOUTH_WEST, Rect(0, center.getNextY(), WEST_WIDTH, SOUTH_HEIGHT));

			
			 
			}
		   virtual void drawChild(InOutGraphics &graphics) {
				for (size_t index  = 0 ; index < BORDER_COUNT; ++index ) {
					if(hasComponent(  Direction(index)) ){


						Component& component =  datas[index]->getComponent ();

						Rect childClippingOnTop = ComponentRegionUtil::getClippingRectOnClient ( component );
						Point childPointOnClient = ComponentRegionUtil::getPointOnClient(component);

						graphics.accessInOutDrawStruct().replaceRegion(childClippingOnTop);
						graphics.accessInOutDrawStruct().replaceStartPoint(childPointOnClient);

						datas[index]->accessComponent ().draw(graphics);

					}
				}
			}		  
		   virtual bool const  empty() const{
				for (size_t index  = 0 ; index < BORDER_COUNT; ++index ) {
					if(hasComponent(  Direction(index)) ){
						return false;
					}
				}

				return true;
			}
			virtual bool const isOkToExit() {
				bool result = true;
				for (size_t index  = 0 ; index < BORDER_COUNT; ++index ) {
					if(hasComponent(  Direction(index)) ){
						result = (result && datas[index]->accessComponent ().isOkToExit());         
							
					}
				}

				return result;
			}
		   virtual  String const toString() const{

				String result;

				

				for (size_t index  = 0 ; index < BORDER_COUNT; ++index ) {


					switch(index) {
		                
						case NORTH :		result += String("NORTH:") ;		break;
						case NORTH_EAST :	result += String("NORTH_EAST:") ;	break;
						case EAST :			result += String("EAST:") ;			break;
						case SOUTH_EAST :	result += String("SOUTH_EAST:") ;	break;
						case SOUTH :		result += String("SOUTH:") ;		break;
						case SOUTH_WEST :	result += String("SOUTH_WEST:") ;	break;
						case WEST :			result += String("WEST:") ;			break;
						case NORTH_WEST :	result += String("NORTH_WEST:") ;	break;
						case CENTER :		result += String("CENTER:") ;		break;


						default:
							assertExp(false);
							break;
					}
					
					
					if(hasComponent(  Direction(index)  ) ){
						result += datas[index]->accessComponent ().toString();
						result += datas[index]->getRect().toString();
						result += String("\n");
							
					}
					else {
						result += String("No Component\n");

					}
				}

				return result;

			}


		private:
		    void safeResite(Direction dir, Rect const & newRect) {
	           
	           if (hasComponent(dir)) {            
					datas[dir]->replaceRect(newRect);
					

					SizeEventSizeChanged eventToComponent(newRect.toSize());
					datas[dir]->accessComponent().invokeSizeListener(eventToComponent );	
				}
				else {
					//ƹ ͵  ʴ´..
					
				}
	       }
		

















	        
	};
}

#endif
//BOARDPANEL_H

