#include "ComponentChainUtil.h"

#include "Component.h"
#include "Container.h"

#include "TopWindowClientComponentMatcher.h"
#include "CreatedComponents.h"
#include "../global/Basic.h"

namespace podo{


	
	unsigned int const ComponentChainUtil::getLevel(Component  const & comp) {	
			
		if(!comp.hasParent()){	
			return 0;							
		}
		else {
			return  1 + getLevel(comp.getParent());
		}

	}

	Component const & ComponentChainUtil::getRoot(Component const & comp,unsigned int const &rootNum) {

		if(rootNum  == getLevel(comp)) {
			return comp;
		}
		else if(rootNum < getLevel(comp)){
			return getRoot(comp.getParent(), rootNum  );                
		}
		else {
			assertExp(false);
			return *Component::createNull();
		}
	}

	Component const & ComponentChainUtil::getCommonParent(Component const & comp1,Component const & comp2) {

		uint32_t levelLeft = getLevel(comp1) ;
		uint32_t levelRight = getLevel(comp2);
		if((levelLeft == levelRight) && (&comp1 == &comp2)) {			
			return comp1;
		}else if((levelLeft == levelRight) && (&comp1 != &comp2)) {
			assertExp(levelLeft > 0);
			
			return getCommonParent(comp1.getParent() ,comp2.getParent());
		}else if(levelLeft > levelRight) {
            return getCommonParent(comp1.getParent(),comp2);
		}else{
			return getCommonParent(comp1,comp2.getParent());
		}		
	}		

	Component const & ComponentChainUtil::getClientComponent(Component const  & comp) {

		if(isClientComponent(comp)) {
			return comp;
		}
		else {
			return getClientComponent(comp.getParent());
		}		
	}


	
	Component  & ComponentChainUtil::accessRoot(Component  & comp,unsigned int const &rootNum) {
		return const_cast<Component &>(getRoot(comp,rootNum) );			
	}

	Component & ComponentChainUtil::accessCommonParent(Component  & comp1,Component  & comp2) {
		return const_cast<Component &>(getCommonParent(comp1,comp2) );
	}		

	Component  & ComponentChainUtil::accessClientComponent(Component  & comp) {
		return const_cast<Component &>(getClientComponent(comp) );
	}
	
	
	bool const ComponentChainUtil::isChainOk(Component const& comp){
	    
	    if(! CreatedComponents::instance().hasComponent(comp)) {
	        return false;	        
	    }
	    
	    
	    Component const& topComp = getClientComponent(comp); 
	    
	    return TopWindowClientComponentMatcher::hasTopWindow(topComp);
	    
	    
	}

}
