#ifndef TOPWINDOW_H
#define TOPWINDOW_H

#include "TopWindow.p"
#include "Component.p"
#include "TopWindowClientComponentMatcher.h"

#include "../global/AggregationAttribute.h"
#include "../system/SystemApi.h"
#include "../global/Rect.p"
#include "../global/Size.h"
#include "../global/Point.p"

#include "painter/InOutGraphics.p"

#include "process/MouseState.p"
#include "../global/KeyboardKey.p"
#include "../system/Message.h"
#include "../global/MouseButton.p"
#include "system/DragData.p"
#include "painter/InOutGraphics.h"


#include "../system/SystemWindow.p"
#include "../system/MessageReceiver.h"


#include "Component.h"


#include "VolatileWindow.p"


namespace podo{


	class TopWindow :public MessageReceiver{
		typedef TopWindow this_t;
	public:


		struct TopWindowFocusListener :public FocusListener {
			this_t & _this;
			TopWindowFocusListener(this_t & this_r):_this(this_r) {

			}


			virtual void focusGained(FocusListenerHaver & sender , FocusEventFocusGained  &event) {
				FocusEventFocusGained eventToComponent;
				_this._currentFocusedComponent->invokeFocusListener(eventToComponent);
			
			}
			virtual void focusLost(FocusListenerHaver & sender ,FocusEventFocusLost  &event) {
				FocusEventFocusLost eventToComponent;
				_this._currentFocusedComponent->invokeFocusListener(eventToComponent);
			
			}

		};


		struct TopWindowSizeListener :public SizeListener {
			this_t & _this;
			TopWindowSizeListener(this_t & this_r):_this(this_r) {

			}


			virtual void sizeChanged(SizeListenerHaver & sender , SizeEventSizeChanged  &event) {
				SizeEventSizeChanged eventToComponent(event._size);
				_this. _topComponent->invokeSizeListener(event );			
			}		

		};

		


	protected:

		SystemWindow * _windowImpl;	

		
		MouseStatePtr  _currentMouseState;
		ClientComponentPtr _topComponent;
		Component * _currentFocusedComponent;



       


protected:

 

		TopWindow();
public:

		virtual ~TopWindow() ;

		void reset() ;
		


	
	public:

		/* note
		* Allwayse have ClientComponent; 
		* 
		*/


		void setClientComponent(ComponentPtr topComponent);
		void setClientComponent(Component* topComponent);

		

	
	public:
		void setFocusedComponent(Component & comp);





		bool const isCreated() const;




		virtual void create() = 0;		
		void  destroy();
		Size const getClientSize() const;
		InOutGraphicsPtr createGraphics() ;
		Point const clientToBackground(Point  const &  pointOnClient ) ;

		
		
		


	public:
		void repaint(Rect const & rect) ;
		void postDestroyRequestMessage();

		

		//ƿ  Լ
	protected:

		void afterCreated();

	public:


		
		virtual void distributeEnforceDestroyMessage();
		virtual ProcessState distributeMouseButtonMessage(bool const & bDowned, MouseButton const& button,
			Point const & pointOnClient,bool bIn);
	    virtual ProcessState distributeKeyMessage(bool const & bDowned, KeyboardKey  const& key);
	    virtual ProcessState distributePaintMessage(PaintDrawStruct & paintDrawStruct);
	    virtual ProcessState distributeMouseMoveMessage(Point const & pointOnClient, bool bIn );
	    virtual ProcessState distributeMouseEnteredMessage();
	    virtual ProcessState distributeMouseExitedMessage();
	  
		virtual ProcessState distributeCloseMessage() ;
	
		
		
		void ChangeMouseState(MouseStatePtr mouseState) ;
		void ChangeMouseStateIllegal(MouseStatePtr mouseState) ;






		





	};

}
#endif
//TOPWINDOW_H

