
#ifndef MOUSELISTENER_H
#define MOUSELISTENER_H




#include "../event/PropertyOldPoint.h"
#include "../event/PropertyCurrentPoint.h"
#include "../event/PropertyDragStarted.h"
#include "../event/PropertyPointDragStarted.h"


#include "../Component.p"
#include "../system/DragData.p"
#include "../../global/Point.h"
#include "../../global/MouseButton.h"


#include "ListenerHaver.h"


#include <vector>




namespace podo{

	


    
	class MouseEvent {
	public:
		enum Mode {	MODE_CAPTURE, 
			MODE_DRAGDROP,
			MODE_NONE
		};


		Mode _mode;

		MouseEvent () :_mode(MODE_CAPTURE){

		}

		virtual ~MouseEvent (){
		}
		

	};


	class MouseEventButtonClicked : public MouseEvent {
	public:

			MouseEventButtonClicked(Point currentPointOnClient,	
				Point clickStartedPointOnClient,	
				MouseButton const & actionButton) 
				: _currentPointOnClient(currentPointOnClient)
				, _clickStartedPointOnClient(clickStartedPointOnClient)
				,_actionButton(actionButton)		{

			}
			

	public:
	private:
			Point _currentPointOnClient;
			Point _clickStartedPointOnClient;

			MouseButton _actionButton;

	public:
			MouseButton const getActionButton() const{

				return _actionButton;
			}

			Point const getCurrentPointOnClient() {
				return _currentPointOnClient;

			}
			Point const getClickStartedPointOnClient() {
				return _clickStartedPointOnClient;

			}
	};



	class MouseEventButtonPressed : public MouseEvent,public PropertyCurrentPoint {
		public:        

				MouseEventButtonPressed(Point const & initCurrentAbsolutePoint,
			MouseButton const & initButton) :_actionButton(initButton),
			PropertyCurrentPoint(initCurrentAbsolutePoint){



		}
			

		private:

				MouseButton _actionButton;
		};




	class MouseEventDraged : public MouseEvent,
		public PropertyCurrentPoint, 
		public PropertyOldPoint,
		public PropertyPointDragStarted {
	public:



		
	MouseEventDraged(Point const & currentPointOnClient,
					Point const & oldPointOnClient,
					MouseButton const &buttonDragStarted,
					Point pointDragStarted,
					bool bOut)
		:_buttonDragStarted(buttonDragStarted),_bOut(bOut),
		PropertyCurrentPoint(currentPointOnClient),
		PropertyOldPoint(oldPointOnClient),
		PropertyPointDragStarted(pointDragStarted){

	}		

	
	private:        
	      
			MouseButton _buttonDragStarted;
			bool _bOut;
	};





		class MouseEventDroped : public MouseEvent {

		private:
			Point _currentAbsolutePoint;
			MouseButton _buttonDragStarted;
			Component * _dragStartedComponent;
			DragDataPtr _dragData;
		public:
			MouseEventDroped( 
				Point const & currentAbsolutePoint,
				Component & dragStartedComponent,
				MouseButton buttonDragStarted
				,	DragDataPtr dragData
				
				)	:_currentAbsolutePoint(currentAbsolutePoint) 
				,_dragStartedComponent(&dragStartedComponent)
				,_buttonDragStarted(buttonDragStarted)
				,_dragData(dragData)			{

			}
			
			

		
	
		};



		
	class MouseEventMouseEntered : public MouseEvent {
	        

	public:        

		MouseEventMouseEntered(){}

			
	};


	class MouseEventMouseExited : public MouseEvent {	        

	public:        

		MouseEventMouseExited(){}
	};



	class MouseEventMouseMoved : public MouseEvent {


	public:

			MouseEventMouseMoved(Point const & initCurrentAbsolutePoint,Point const & initOldAbsolutePoint)
				:_currentAbsolutePoint(initCurrentAbsolutePoint), 
				_oldAbsolutePoint(initOldAbsolutePoint),MouseEvent(){

			}
			

	private:        
			Point _currentAbsolutePoint;
			Point _oldAbsolutePoint;
	private:
	};

	class MouseListenerHaver;
	class MouseListener;
	typedef boost::shared_ptr<MouseListener> MouseListenerPtr; 
	class MouseListener {
		public:		
		virtual ~MouseListener();
		
		// ۿ ϶ ߻ ϴ ޼          
		virtual void mousePressed(MouseListenerHaver & sender, MouseEventButtonPressed & event) = 0;
		virtual void mouseClicked(MouseListenerHaver & sender,MouseEventButtonClicked &event) = 0;
		virtual void mouseDroped(MouseListenerHaver & sender,MouseEventDroped &event) = 0;
		//  ߻ ϴ ޼		
		virtual void mouseDraged(MouseListenerHaver & sender,MouseEventDraged &event) = 0;
		virtual void mouseMoved(MouseListenerHaver & sender,MouseEventMouseMoved &event) = 0;
		virtual void mouseExited(MouseListenerHaver & sender,MouseEventMouseExited &event) = 0;
		virtual void mouseEntered(MouseListenerHaver & sender,MouseEventMouseEntered &event) = 0;

		
		static MouseListenerPtr createNull() ;
//end Func
	};


	class MouseAdapter : public MouseListener {




		
		// ۿ ϶ ߻ ϴ ޼          
		virtual void mousePressed(MouseListenerHaver & sender, MouseEventButtonPressed & event) {		}
		virtual void mouseClicked(MouseListenerHaver & sender,MouseEventButtonClicked &event) {}
		virtual void mouseDroped(MouseListenerHaver & sender,MouseEventDroped &event) {}
		//  ߻ ϴ ޼		
		virtual void mouseDraged(MouseListenerHaver & sender,MouseEventDraged &event) {}
		virtual void mouseMoved(MouseListenerHaver & sender,MouseEventMouseMoved &event) {}
		virtual void mouseExited(MouseListenerHaver & sender,MouseEventMouseExited &event) {}
		virtual void mouseEntered(MouseListenerHaver & sender,MouseEventMouseEntered &event) {}

		



	};


	

		//template <typename THIS_LISTENER>
		class MouseThisListener :public MouseListener {

		private:
			MouseListener & _this;
		public:

			virtual ~MouseThisListener(){

			}
			MouseThisListener(MouseListener& thislistener) :_this(thislistener){

			}
			
			// ۿ ϶ ߻ ϴ ޼          
			virtual void mousePressed(MouseListenerHaver & sender, MouseEventButtonPressed & event) {
				_this.mousePressed(sender,event);
			}
			virtual void mouseClicked(MouseListenerHaver & sender, MouseEventButtonClicked  &event) {
				_this.mouseClicked(sender,event);
				
			}
			virtual void mouseDroped(MouseListenerHaver & sender, MouseEventDroped  &event) {
				_this.mouseDroped(sender,event);
			}
			//  ߻ ϴ ޼		
			virtual void mouseDraged(MouseListenerHaver & sender, MouseEventDraged 	&event){
				_this.mouseDraged(sender,event);					
			}
			virtual void mouseMoved(MouseListenerHaver & sender, MouseEventMouseMoved  &event) {
                _this.mouseMoved(sender,event);
			}
			virtual void mouseExited(MouseListenerHaver & sender, MouseEventMouseExited  &event) {
                _this.mouseExited(sender,event);					
			}
			virtual void mouseEntered(MouseListenerHaver & sender, MouseEventMouseEntered  &event) {
				_this.mouseEntered(sender, event);	                
			}
			
		};


		class MouseMacroListener : public MouseListener {
		        
		private:
			MouseListenerPtr _before;
			MouseListenerPtr _after;
		public:

			template <typename MouseListenerPPP>
			MouseMacroListener(MouseListenerPPP before, MouseListenerPPP after) : _before(before), _after(after) {  	}



			// ۿ ϶ ߻ ϴ ޼          
			virtual void mousePressed(MouseListenerHaver & sender, MouseEventButtonPressed & event) {
				_before->mousePressed(sender,event);
				_after->mousePressed(sender,event);
			}
			virtual void mouseClicked(MouseListenerHaver & sender, MouseEventButtonClicked  &event) {
				_before->mouseClicked(sender,event);
				_after->mouseClicked(sender,event);
				
			}
			virtual void mouseDroped(MouseListenerHaver & sender, MouseEventDroped  &event) {
				_before->mouseDroped(sender,event);
				_after->mouseDroped(sender,event);
			}
			//  ߻ ϴ ޼		
			virtual void mouseDraged(MouseListenerHaver & sender, MouseEventDraged 	&event){
				_before->mouseDraged(sender,event);
				_after->mouseDraged(sender,event);					
			}
			virtual void mouseMoved(MouseListenerHaver & sender, MouseEventMouseMoved  &event) {
                _before->mouseMoved(sender,event);
				_after->mouseMoved(sender,event);
			}
			virtual void mouseExited(MouseListenerHaver & sender, MouseEventMouseExited  &event) {
                _before->mouseExited(sender,event);
				_after->mouseExited(sender,event);					
			}
			virtual void mouseEntered(MouseListenerHaver & sender, MouseEventMouseEntered  &event) {
				_before->mouseEntered(sender,event);	
				_after->mouseEntered(sender,event);                
			}
			
		
		};



    class MouseListenerHaver {

	private:        
		ListenerHaver<MouseListener,MouseMacroListener > _listenerHaver;

	

	public:

		MouseListenerHaver() {

		}

	

		void setMouseListener(MouseListener* mouseListener) {
			_listenerHaver.setListener(mouseListener);
		}

		
		void addMouseListener(MouseListener* mouseListener) {
			_listenerHaver.addListener(mouseListener);
		}

		void setMouseListener(MouseListenerPtr mouseListener) {
			_listenerHaver.setListener(mouseListener);
		}
		
		void addMouseListener(MouseListenerPtr mouseListener) {
			_listenerHaver.addListener(mouseListener);
		}
		



		void invokeMouseListener( MouseEventButtonPressed & event) {

			


			typedef MouseListener TT_Listener;
			typedef MouseListenerHaver TT_ListenerHaver;
			typedef MouseEventButtonPressed TT_Event;


			typedef void ( TT_Listener::*Func)(TT_ListenerHaver &, TT_Event  &);
			_listenerHaver.invokeListener<Func,TT_ListenerHaver &, TT_Event &>(&TT_Listener::mousePressed ,*this,event);
		}
		void invokeMouseListener(MouseEventButtonClicked &event) {
			
			typedef MouseListener TT_Listener;
			typedef MouseListenerHaver TT_ListenerHaver;
			typedef MouseEventButtonClicked TT_Event;


			typedef void ( TT_Listener::*Func)(TT_ListenerHaver &, TT_Event  &);
			_listenerHaver.invokeListener<Func,TT_ListenerHaver &, TT_Event &>(&TT_Listener::mouseClicked ,*this,event);
		}
		void invokeMouseListener(MouseEventDroped &event) {
			

			typedef MouseListener TT_Listener;
			typedef MouseListenerHaver TT_ListenerHaver;
			typedef MouseEventDroped TT_Event;


			typedef void ( TT_Listener::*Func)(TT_ListenerHaver &, TT_Event  &);
			_listenerHaver.invokeListener<Func,TT_ListenerHaver &, TT_Event &>(&TT_Listener::mouseDroped ,*this,event);
		}
		void invokeMouseListener(MouseEventDraged &event) {
			

			typedef MouseListener TT_Listener;
			typedef MouseListenerHaver TT_ListenerHaver;
			typedef MouseEventDraged TT_Event;


			typedef void ( TT_Listener::*Func)(TT_ListenerHaver &, TT_Event  &);
			_listenerHaver.invokeListener<Func,TT_ListenerHaver &, TT_Event &>(&TT_Listener::mouseDraged ,*this,event);
		}
		void invokeMouseListener(MouseEventMouseMoved &event) {
			


			typedef MouseListener TT_Listener;
			typedef MouseListenerHaver TT_ListenerHaver;
			typedef MouseEventMouseMoved TT_Event;


			typedef void ( TT_Listener::*Func)(TT_ListenerHaver &, TT_Event  &);
			_listenerHaver.invokeListener<Func,TT_ListenerHaver &, TT_Event &>(&TT_Listener::mouseMoved ,*this,event);
		}
		void invokeMouseListener(MouseEventMouseExited &event) {

			


			typedef MouseListener TT_Listener;
			typedef MouseListenerHaver TT_ListenerHaver;
			typedef MouseEventMouseExited TT_Event;


			typedef void ( TT_Listener::*Func)(TT_ListenerHaver &, TT_Event  &);
			_listenerHaver.invokeListener<Func,TT_ListenerHaver &, TT_Event &>(&TT_Listener::mouseExited ,*this,event);
		}
		void invokeMouseListener(MouseEventMouseEntered &event) {

		

			typedef MouseListener TT_Listener;
			typedef MouseListenerHaver TT_ListenerHaver;
			typedef MouseEventMouseEntered TT_Event;


			typedef void ( TT_Listener::*Func)(TT_ListenerHaver &, TT_Event  &);
			_listenerHaver.invokeListener<Func,TT_ListenerHaver &, TT_Event &>(&TT_Listener::mouseEntered ,*this,event);
		}


		virtual ~MouseListenerHaver () {		}


  
    };

		


}



#endif
//MOUSELISTENER_H

